import os
from temporalio.client import Client


API_KEY_ENV = "API_KEY"
API_KEY_MD_KEY = "durable_api_key"


async def get_client(api_key: str=None) -> Client:
    if not api_key and API_KEY_ENV in os.environ:
        api_key = os.environ[API_KEY_ENV]

    if not api_key:
        raise KeyError("Durable SDK could not find API key")

    # TODO: Remove specific namespace
    return await Client.connect("localhost:7233", namespace='65b1872cdb6ba5ac49fc5274', rpc_metadata={
        API_KEY_MD_KEY: api_key
    })



import typer
from pydurable.cli.config import API_URL
from pydurable.cli.login import get_authenticated_session


app = typer.Typer()


@app.command('register')
def register_func(name: str):
    session = get_authenticated_session()
    response = session.post(f'{API_URL}/func/register/{name}').json()
    typer.echo(response)


@app.command('list')
def list_funcs():
    session = get_authenticated_session()
    funcs = session.get(f'{API_URL}/func/').json()
    for i, func in enumerate(funcs):
        typer.echo(f'{i}. "{func}"')


@app.command('delete')
def delete_func(name: str):
    session = get_authenticated_session()
    response = session.delete(f'{API_URL}/func/{name}').json()
    typer.echo(response)



from temporalio.worker import Worker

from pydurable.worker.client import get_client
from pydurable.worker.wait_for_exit import wait_for_exit


async def run(workflows, activities, api_key: str=None, context=None, function_name=None):
    if not function_name and context:
        function_name = context.function_name

    if not function_name:
        raise KeyError('Run function called without AWS Lambda context or a function name. Please provide one.')

    client = await get_client(api_key)

    async with Worker(
        client,
        task_queue=function_name,
        workflows=workflows,
        activities=activities,
    ):
       print('Started Worker')
       await wait_for_exit(client, function_name)



import typer
from pydurable.cli.login import get_authenticated_session

from pydurable.cli.config import API_URL


app = typer.Typer()


@app.command('list')
def list_apikeys():
    session = get_authenticated_session()
    response = session.get(f'{API_URL}/apikey/')
    keys = response.json()
    if not keys:
        typer.echo('No API KEYS created yet.')
        return
    
    typer.echo(f'API KEYS:')
    for i, key in enumerate(keys):
        typer.echo(f' {i}. "{key}"')


@app.command('create')
def create_apikey():
    session = get_authenticated_session()
    res = session.post(f'{API_URL}/apikey/')
    key = res.json()
    typer.echo(f'Your new API KEY is: {key}.')


@app.command('delete')
def list_apikeys(index: int):
    session = get_authenticated_session()
    response = session.delete(f'{API_URL}/apikey/{index}')
    if not response.ok:
        typer.echo('Failed', response.json())
        return
    
    typer.echo(f'Deleted API KEY {index}.')



import asyncio
from datetime import datetime, timedelta
import traceback
from typing import List
from temporalio.client import Client
from temporalio.api.workflowservice.v1 import (
    GetWorkflowExecutionHistoryReverseRequest,
    GetWorkflowExecutionHistoryReverseResponse,
    ListOpenWorkflowExecutionsRequest,
    ListOpenWorkflowExecutionsResponse
)
from temporalio.api.common.v1 import WorkflowExecution
from temporalio.api.history.v1 import History
from temporalio.api.enums.v1 import HistoryEventFilterType, EventType
from temporalio.api.workflow.v1 import WorkflowExecutionInfo


async def wait_for_exit(client: Client, task_queue):
    while True:
        try:
            if await should_stop(client, task_queue):
                print('Exiting')
                return
            
            print('Not Exiting')
        except Exception as e:
            print(f'Error while checking for stopping condition {traceback.format_exc()}')
        await asyncio.sleep(1)


async def should_stop(client, task_queue):
    namespace = client.config()['namespace']
    open_wf_response: ListOpenWorkflowExecutionsResponse = \
        await client.service_client.workflow_service.list_open_workflow_executions(
            ListOpenWorkflowExecutionsRequest(
                namespace=namespace
            )
        )
    
    executions_info: List[WorkflowExecutionInfo] = \
        filter(lambda execinfo: execinfo.task_queue == task_queue, open_wf_response.executions)
    
    for execution_info in executions_info:
        wf_history_response: GetWorkflowExecutionHistoryReverseResponse = \
            await client.service_client.workflow_service.get_workflow_execution_history_reverse(
                GetWorkflowExecutionHistoryReverseRequest(
                    namespace=namespace,
                    execution=WorkflowExecution(workflow_id=execution_info.execution.workflow_id),
                    maximum_page_size=1
                )
            )
        
        history: History = wf_history_response.history
        if len(history.events) == 0:
            continue

        last_event = history.events[0]
        if last_event.event_type == EventType.EVENT_TYPE_TIMER_STARTED:
            event_time = last_event.event_time.ToDatetime()
            timer_seconds = last_event.timer_started_event_attributes.start_to_fire_timeout.ToTimedelta()
            end_time = event_time + timer_seconds
            now = datetime.now()
            if (end_time - now).total_seconds() > 10:
                continue

        return False

    return True



import typer
from pydurable.cli.config import API_URL
from pydurable.cli.login import get_authenticated_session


app = typer.Typer()


@app.command('info')
def info():
    session = get_authenticated_session()
    data = session.get(f'{API_URL}/aws/info').json()
    typer.echo(data)


@app.command('update-role')
def update_role(role: str):
    session = get_authenticated_session()
    response = session.put(f'{API_URL}/aws/role/{role}').json()
    typer.echo(response)


@app.command('update-account')
def update_account(account_id: str):
    session = get_authenticated_session()
    response = session.put(f'{API_URL}/aws/account/{account_id}').json()
    typer.echo(response)



import datetime
import json
import logging
from pathlib import Path
import threading
from typing import Any
import requests
import typer
from http.server import BaseHTTPRequestHandler, HTTPServer
from urllib.parse import urlparse, parse_qs

from pydurable.cli.config import API_URL


LOGIN_PORT = 54987
CLIENT_ID = '685954567384-9otb323plmau5iutge14kb4l7aei889n.apps.googleusercontent.com'


def login():
    code = None

    class RedirectHandler(BaseHTTPRequestHandler):
        def do_GET(self):
            parsed_url = urlparse(self.path)
            query_params = parse_qs(parsed_url.query)
            nonlocal code
            try:
                code = query_params['code'][0]
            except KeyError:
                pass
            self.send_response(200)
            self.send_header("Content-type", "text/text")
            self.end_headers()
            response_message = "Logged In to Durable"
            self.wfile.write(response_message.encode('utf-8'))
            if code:
                threading.Thread(target=self.server.shutdown).start()
        
        def log_message(self, format: str, *args: Any) -> None:
            pass

    logging.getLogger('http.server').setLevel(logging.WARNING)
    httpd = HTTPServer(("127.0.0.1", LOGIN_PORT), RedirectHandler)
    server_thread = threading.Thread(target=httpd.serve_forever)
    server_thread.start()
    try:
        typer.launch('https://accounts.google.com/o/oauth2/v2/auth?response_type=code&scope=https://www.googleapis.com/auth/userinfo.email'
                     f'&client_id={CLIENT_ID}&redirect_uri=http://127.0.0.1:{LOGIN_PORT}')
        server_thread.join()
        print("Logged In")
    except KeyboardInterrupt:
        print("Stopping.")
    else:
        httpd.shutdown()

    tokens = requests.get(f"{API_URL}/login/google", params={
        "code": code
    }).json()
    save_tokens(tokens)


DURABLE_DIR = Path.home() / ".durable"
TOKEN_PATH = DURABLE_DIR / "token"


def save_tokens(tokens):
    DURABLE_DIR.mkdir(parents=True, exist_ok=True)
    with TOKEN_PATH.open(mode='w') as token_file:
        json.dump(tokens, token_file)


def get_tokens():
    with TOKEN_PATH.open(mode='r') as token_file:
        return json.load(token_file)


def get_authenticated_session():
    tokens = get_tokens()
    session = requests.Session()
    session.headers.update({'Authorization': f'Bearer {tokens["access_token"]}'})
    return session



import asyncio
from dataclasses import asdict, is_dataclass
from typing import Any, Optional, Type, Union

from temporalio import activity, workflow
from temporalio.worker import (
    ActivityInboundInterceptor,
    ExecuteActivityInput,
    ExecuteWorkflowInput,
    Interceptor,
    WorkflowInboundInterceptor,
    WorkflowInterceptorClassInput,
)


class _ActivityInboundInterceptor(ActivityInboundInterceptor):
    async def execute_activity(self, input: ExecuteActivityInput) -> Any:
        activity_info = activity.info()
        print(f'Executing activity {activity_info}.')
        return await super().execute_activity(input)


class _WorkflowInterceptor(WorkflowInboundInterceptor):
    async def execute_workflow(self, input: ExecuteWorkflowInput) -> Any:
        workflow_info = workflow.info()
        print(f'Executing workflow {workflow_info}.')
        return await super().execute_workflow(input)


class MyInterceptor(Interceptor):
    def intercept_activity(self, next: ActivityInboundInterceptor) -> ActivityInboundInterceptor:
        return _ActivityInboundInterceptor(super().intercept_activity(next))

    def workflow_interceptor_class(self, input: WorkflowInterceptorClassInput) -> Optional[Type[WorkflowInboundInterceptor]]:
        return _WorkflowInterceptor
    
    async def wait(self):
        await asyncio.sleep(60 * 15)


# def has_tasks_in_queue(workflow_client, task_queue):
#     request = DescribeTaskQueueRequest(
#         namespace="default",
#         task_queue=TaskQueue(name=task_queue, kind=TaskQueueKind.TASK_QUEUE_KIND_NORMAL)
#     )
#     response = workflow_client.service_stub.DescribeTaskQueue(request)
#     return response.poller_info.possible_queries_count > 0



import typer
from pydurable.cli.login import login
from pydurable.cli.apikey import app as apikey
from pydurable.cli.aws import app as aws
from pydurable.cli.func import app as func


LOGIN_PORT = 54987
CLIENT_ID = '685954567384-9otb323plmau5iutge14kb4l7aei889n.apps.googleusercontent.com'
API = 'http://localhost:8000'


app = typer.Typer()
app.command()(login)
app.add_typer(apikey, name='keys')
app.add_typer(aws, name='aws')
app.add_typer(func, name='func')


def main():
    app()
    

if __name__ == '__main__':
    app()
